@@ -5,7 +5,7 @@ format.
 """
 from rdflib.term import Literal
 from rdflib.serializer import Serializer
-from rdflib.py3compat import b
+from six import b
 
 import warnings
 import codecs
@@ -20,12 +20,12 @@ class NTSerializer(Serializer):
 
     def __init__(self, store):
         Serializer.__init__(self, store)
-        self.encoding = 'ascii' # n-triples are ascii encoded
+        self.encoding = 'ascii'  # n-triples are ascii encoded
 
     def serialize(self, stream, base=None, encoding=None, **args):
         if base is not None:
             warnings.warn("NTSerializer does not support base.")
-        if encoding is not None:
+        if encoding is not None and encoding.lower() != self.encoding.lower():
             warnings.warn("NTSerializer does not use custom encoding.")
         encoding = self.encoding
         for triple in self.store:
@@ -41,17 +41,17 @@ class NT11Serializer(NTSerializer):
     """
 
     def __init__(self, store):
-        Serializer.__init__(self, store) # default to utf-8
+        Serializer.__init__(self, store)  # default to utf-8
 
 
 def _nt_row(triple):
     if isinstance(triple[2], Literal):
-        return "%s %s %s .\n" % (
+        return u"%s %s %s .\n" % (
             triple[0].n3(),
             triple[1].n3(),
             _quoteLiteral(triple[2]))
     else:
-        return "%s %s %s .\n" % (triple[0].n3(),
+        return u"%s %s %s .\n" % (triple[0].n3(),
                                   triple[1].n3(),
                                   triple[2].n3())
 
@@ -79,18 +79,19 @@ def _quote_encode(l):
         .replace('"', '\\"')\
         .replace('\r', '\\r')
 
-def _nt_unicode_error_resolver(err):
 
+def _nt_unicode_error_resolver(err):
     """
     Do unicode char replaces as defined in https://www.w3.org/TR/2004/REC-rdf-testcases-20040210/#ntrip_strings
     """
 
     def _replace_single(c):
         c = ord(c)
-        fmt = '\\u%04X' if c <= 0xFFFF else '\\U%08X'
+        fmt = u'\\u%04X' if c <= 0xFFFF else u'\\U%08X'
         return fmt % c
 
     string = err.object[err.start:err.end]
-    return ( "".join( _replace_single(c) for c in string ), err.end )
+    return ("".join(_replace_single(c) for c in string), err.end)
+
 
 codecs.register_error('_rdflib_nt_escape', _nt_unicode_error_resolver)
