@@ -6,8 +6,8 @@ See <http://www.w3.org/TR/trig/> for syntax specification.
 from collections import defaultdict
 
 from rdflib.plugins.serializers.turtle import TurtleSerializer, _GEN_QNAME_FOR_DT, VERB
-
 from rdflib.term import BNode, Literal
+from six import b
 
 __all__ = ['TrigSerializer']
 
@@ -15,7 +15,7 @@ __all__ = ['TrigSerializer']
 class TrigSerializer(TurtleSerializer):
 
     short_name = "trig"
-    indentString = 4 * ' '
+    indentString = 4 * u' '
 
     def __init__(self, store):
         if store.context_aware:
@@ -39,7 +39,7 @@ class TrigSerializer(TurtleSerializer):
             for triple in context:
                 self.preprocessTriple(triple)
 
-            self._contexts[context]=(self.orderSubjects(), self._subjects, self._references)
+            self._contexts[context] = (self.orderSubjects(), self._subjects, self._references)
 
     def reset(self):
         super(TrigSerializer, self).reset()
@@ -49,7 +49,11 @@ class TrigSerializer(TurtleSerializer):
                   spacious=None, **args):
         self.reset()
         self.stream = stream
-        self.base = base
+        # if base is given here, use that, if not and a base is set for the graph use that
+        if base is not None:
+            self.base = base
+        elif self.store.base is not None:
+            self.base = self.store.base
 
         if spacious is not None:
             self._spacious = spacious
@@ -59,15 +63,16 @@ class TrigSerializer(TurtleSerializer):
         self.startDocument()
 
         firstTime = True
-        for store, (ordered_subjects, subjects, ref) in list(self._contexts.items()):
-            if not ordered_subjects: continue
+        for store, (ordered_subjects, subjects, ref) in self._contexts.items():
+            if not ordered_subjects:
+                continue
 
             self._references = ref
             self._serialized = {}
             self.store = store
             self._subjects = subjects
 
-            if self.default_context and store.identifier==self.default_context:
+            if self.default_context and store.identifier == self.default_context:
                 self.write(self.indent() + '\n{')
             else:
                 if isinstance(store.identifier, BNode):
@@ -90,4 +95,4 @@ class TrigSerializer(TurtleSerializer):
             self.write('}\n')
 
         self.endDocument()
-        stream.write("\n".encode('ascii'))
+        stream.write(b("\n"))
