@@ -6,10 +6,14 @@ http://www.w3.org/TR/sparql11-query/#sparqlQuery
 
 """
 
+from __future__ import print_function
+
 import functools
 import operator
 import collections
 
+from functools import reduce
+
 from rdflib import Literal, Variable, URIRef, BNode
 
 from rdflib.plugins.sparql.sparql import Prologue, Query
@@ -20,7 +24,6 @@ from rdflib.paths import (
     InvPath, AlternativePath, SequencePath, MulPath, NegatedPath)
 
 from pyparsing import ParseResults
-from functools import reduce
 
 
 # ---------------------------
@@ -64,9 +67,11 @@ def Filter(expr, p):
 def Extend(p, expr, var):
     return CompValue('Extend', p=p, expr=expr, var=var)
 
+
 def Values(res):
     return CompValue('values', res=res)
 
+
 def Project(p, PV):
     return CompValue('Project', p=p, PV=PV)
 
@@ -76,9 +81,8 @@ def Group(p, expr=None):
 
 
 def _knownTerms(triple, varsknown, varscount):
-    return (len([_f for _f in (x not in varsknown and
-                              isinstance(
-                                  x, (Variable, BNode)) for x in triple) if _f]),
+    return (len([x for x in triple if x not in varsknown and
+                 isinstance(x, (Variable, BNode))]),
             -sum(varscount.get(x, 0) for x in triple),
             not isinstance(triple[2], Literal),
             )
@@ -115,8 +119,8 @@ def reorderTriples(l):
                        1], varsknown, varscount), x[1]) for x in l[i:])
         t = l[i][0][0]  # top block has this many terms bound
         j = 0
-        while i+j < len(l) and l[i+j][0][0] == t:
-            for c in l[i+j][1]:
+        while i + j < len(l) and l[i + j][0][0] == t:
+            for c in l[i + j][1]:
                 _addvar(c, varsknown)
             j += 1
         i += 1
@@ -130,7 +134,7 @@ def triples(l):
     if (len(l) % 3) != 0:
         raise Exception('these aint triples')
     return reorderTriples((l[x], l[x + 1], l[x + 2])
-                          for x in list(range(0, len(l), 3)))
+                          for x in range(0, len(l), 3))
 
 
 def translatePName(p, prologue):
@@ -148,7 +152,6 @@ def translatePName(p, prologue):
 
 
 def translatePath(p):
-
     """
     Translate PropertyPath expressions
     """
@@ -194,7 +197,6 @@ def translatePath(p):
 
 
 def translateExists(e):
-
     """
     Translate the graph pattern used by EXISTS and NOT EXISTS
     http://www.w3.org/TR/sparql11-query/#sparqlCollectFilters
@@ -214,7 +216,6 @@ def translateExists(e):
 
 
 def collectAndRemoveFilters(parts):
-
     """
 
     FILTER expressions apply to the whole group graph pattern in which
@@ -365,7 +366,7 @@ def _traverseAgg(e, visitor=lambda n, v: None):
 
     elif isinstance(e, CompValue):
         for k, val in e.items():
-            if val != None:
+            if val is not None:
                 res.append(_traverseAgg(val, visitor))
 
     return visitor(e, res)
@@ -442,7 +443,8 @@ def _addVars(x, children):
             x["_vars"] = set()
         elif x.name == "Extend":
             # vars only used in the expr for a bind should not be included
-            x["_vars"] = reduce(operator.or_, [ child for child,part in zip(children,x) if part!='expr' ], set())
+            x["_vars"] = reduce(operator.or_, [child for child,
+                                               part in zip(children, x) if part != 'expr'], set())
 
         else:
             x["_vars"] = set(reduce(operator.or_, children, set()))
@@ -488,7 +490,6 @@ def translateAggregates(q, M):
                 v.expr = traverse(v.expr, functools.partial(_sample, v=v.evar))
                 v.expr = traverse(v.expr, functools.partial(_aggs, A=A))
 
-
     # having clause
     if traverse(q.having, _hasAggregate, complete=False):
         q.having = traverse(q.having, _sample)
@@ -526,7 +527,7 @@ def translateValues(v):
             res.append({v.var[0]: val})
     else:
         for vals in v.value:
-            res.append(dict(list(zip(v.var, vals))))
+            res.append(dict(zip(v.var, vals)))
 
     return Values(res)
 
@@ -604,7 +605,7 @@ def translate(q):
     # ORDER BY
     if q.orderby:
         M = OrderBy(M, [CompValue('OrderCondition', expr=c.expr,
-                    order=c.order) for c in q.orderby.condition])
+                                  order=c.order) for c in q.orderby.condition])
 
     # PROJECT
     M = Project(M, PV)
@@ -617,10 +618,10 @@ def translate(q):
 
     if q.limitoffset:
         offset = 0
-        if q.limitoffset.offset != None:
+        if q.limitoffset.offset is not None:
             offset = q.limitoffset.offset.toPython()
 
-        if q.limitoffset.limit != None:
+        if q.limitoffset.limit is not None:
             M = CompValue('Slice', p=M, start=offset,
                           length=q.limitoffset.limit.toPython())
         else:
@@ -643,7 +644,6 @@ def simplify(n):
 
 
 def analyse(n, children):
-
     """
     Some things can be lazily joined.
     This propegates whether they can up the tree
@@ -805,6 +805,7 @@ def pprintAlgebra(q):
         for x in q:
             pp(x)
 
+
 if __name__ == '__main__':
     import sys
     from rdflib.plugins.sparql import parser
@@ -818,4 +819,4 @@ if __name__ == '__main__':
     pq = parser.parseQuery(q)
     print(pq)
     tq = translateQuery(pq)
-    print(pprintAlgebra(tq))
+    pprintAlgebra(tq)
