@@ -56,7 +56,7 @@ class AuditableStore(Store):
             context = context.__class__(self.store, context.identifier) if context is not None else None
             ctxId = context.identifier if context is not None else None
             if list(self.store.triples(triple, context)):
-                return # triple already in store, do nothing
+                return  # triple already in store, do nothing
             self.reverseOps.append((s, p, o, ctxId, 'remove'))
             try:
                 self.reverseOps.remove((s, p, o, ctxId, 'add'))
@@ -64,8 +64,8 @@ class AuditableStore(Store):
                 pass
             self.store.add((s, p, o), context, quoted)
 
-    def remove(self, xxx_todo_changeme, context=None):
-        (subject, predicate, object_) = xxx_todo_changeme
+    def remove(self, spo, context=None):
+        subject, predicate, object_ = spo
         lock = destructiveOpLocks['remove']
         lock = lock if lock else threading.RLock()
         with lock:
@@ -88,7 +88,7 @@ class AuditableStore(Store):
                             self.reverseOps.append((s, p, o, ctx.identifier, 'add'))
             else:
                 if not list(self.triples((subject, predicate, object_), context)):
-                    return # triple not present in store, do nothing
+                    return  # triple not present in store, do nothing
                 try:
                     self.reverseOps.remove((subject, predicate, object_, ctxId, 'remove'))
                 except ValueError:
